from player import *
from map import *
from level import *


class Game:
    def __init__(self):
        self.stage = 21
        self.dead = False
        self.fps_cap = 40
        self.current_map = None
        self.current_goal = 0
        self.target = None
        self.lives = 0
        self.max_lives = 0
        self.jsons = (map1_json, map2_json, map3_json, map4_json, map5_json)
        self.player = None
        self.load_map(map1_json)
        self.starting = True
        self.starting_progress = 100

    def load_map(self, map_json):
        self.current_map = Map()
        self.current_map.load(map_json)
        self.current_goal = map_json["goal"]
        self.lives = 0
        self.max_lives = 0
        for npc in self.current_map.npcs:
            if npc.type == 0:
                self.target = npc
            elif npc.type == 2:
                self.lives += 1
                self.max_lives += 1
        self.player = Player(self.current_map)

    def draw_starting(self):
        pygame.draw.rect(screen, (255, 255, 255),
                         pygame.Rect(400 - self.starting_progress * 8 / 2, 250 - self.starting_progress * 5 / 2,
                                     self.starting_progress * 8, self.starting_progress * 5))
        self.starting_progress -= 7
        if self.starting_progress <= 0:
            self.starting = False

    def update(self):
        if 0 < self.stage < 10:
            self.current_map.update()
            self.player.update()
            self.lives = 0
            for npc in self.current_map.npcs:
                if npc.type == 2 and npc.isActive:
                    self.lives += 1
                elif npc.type == 0 and not npc.isActive:
                    ses[4].play_once()
                    if self.stage == 5:
                        self.stage = -1
                    else:
                        self.stage += 21
            if self.lives <= 0 or self.target.pos[0] > self.current_goal:
                self.stage += 10
                ses[1].play_once()

    def draw(self):
        screen.fill((210, 210, 210))
        if self.stage < 0:
            screen.blit(texture_lib["win"], (0, 0))
        elif self.stage < 10:
            self.player.draw()
            self.current_map.draw()
        elif self.stage < 20:
            screen.blit(texture_lib["over"], (0, 0))
        else:
            screen.blit(texture_lib["stage" + str(self.stage-20)], (0, 0))
        screen.blit(texture_lib["progress_bar"], (200, 20))
        pos1 = self.player.pos[0]/self.current_goal * 400 + 200
        pos2 = self.target.pos[0]/self.current_goal * 400 + 200
        if pos1 < 200:
            pos1 = 200
        elif pos1 > 600:
            pos1 = 600
        if pos2 < 200:
            pos2 = 200
        elif pos2 > 600:
            pos2 = 600
        screen.blit(texture_lib["cop_head"], (pos1-20, 20))
        screen.blit(texture_lib["target_head"], (pos2-20, 20))
        for i in range(self.max_lives):
            if i < self.lives:
                screen.blit(texture_lib["civilian_head"], (150 - i * 40, 18))
            else:
                screen.blit(texture_lib["civilian_dead"], (150 - i * 40, 18))

    def handle_event(self, event):
        if self.stage < 0:
            pass
        elif self.stage < 10:
            self.player.handle_event(event)
        elif self.stage < 20:
            if event.type == pygame.KEYUP and event.key == pygame.K_p:
                self.stage = self.stage + 10
        else:
            if event.type == pygame.KEYUP and event.key == pygame.K_p:
                self.stage = self.stage - 20
                self.load_map(self.jsons[self.stage - 1])

    def start(self):
        self.update()
        while not self.dead:
            ENV['delta_time'] = clock.tick(self.fps_cap)/1000
            for event in pygame.event.get():
                handle_common_event(event)
                self.handle_event(event)
            if self.starting:
                self.draw()
                self.draw_starting()
            else:
                self.update()
                self.draw()
            pygame.display.flip()
