from config import *
from npc import *


class Platform:
    def __init__(self, x, y, w, h):
        self.x = x
        self.y = y
        self.w = w
        self.h = h
        self.rect = pygame.Rect(x, y, w, h)

    def get_rect(self):
        return pygame.Rect(self.x - ENV["global_offset"][0] + DEFAULT_CENTER[0],
                           self.y - ENV["global_offset"][1] + DEFAULT_CENTER[1],
                           self.w, self.h)

    def draw(self):
        pygame.draw.rect(screen, (100, 100, 100), self.get_rect())

    def collide_point(self, point):
        return self.rect.collidepoint(point)

    def collide_rect(self, r2):
        return self.rect.colliderect(r2)

    def collide_with(self, r2):
        r1 = self.rect
        dx = (r1.x + r1.w / 2) - (r2.x + r2.w / 2)
        dy = (r1.y + r1.h / 2) - (r2.y + r2.h / 2)
        width = (r1.w + r2.w) / 2
        height = (r1.h + r2.h) / 2

        cross_width = width * dy
        cross_height = height * dx

        if abs(dx) <= width and abs(dy) <= height:
            if cross_width > cross_height:
                if cross_width > -cross_height:
                    return "bottom"
                else:
                    return "left"
            else:
                if cross_width > -cross_height:
                    return "right"
                else:
                    return "top"
        return ""


class Map:
    def __init__(self):
        # Currently player can access world, but npcs cannot access player
        self.platforms = []
        self.npcs = []
        self.animations = []
        self.lives = 3

    def load(self, json):
        for platform in json["platforms"]:
            self.platforms.append(Platform(platform[0], platform[1], platform[2], platform[3]))
        for aNpc in json['npcs']:
            self.npcs.append(Npc(self, aNpc[0], [aNpc[1][0], aNpc[1][1]]))

    def draw(self):
        for platform in self.platforms:
            platform.draw()
        for aNpc in self.npcs:
            if aNpc.isActive:
                aNpc.draw()
        for i in range(len(self.animations) - 1, -1, -1):
            self.animations[i].draw(ENV["global_offset"])
            if self.animations[i].is_dead():
                del self.animations[i]
        pygame.draw.rect(screen, (50, 50, 50),
                         pygame.Rect(0, GROUND_LEVEL - ENV["global_offset"][1] + DEFAULT_CENTER[1] + PLAYER_SIZE[1],
                                     SCREEN_WIDTH, SCREEN_HEIGHT))

    def update(self):
        if self.lives == 0:
            print("Level failed")  # TODO: add level failed display.
        for aNpc in self.npcs:
            if aNpc.isActive:
                aNpc.update()

    def collide_rect(self, player):
        for platform in self.platforms:
            if platform.collide_rect(player.get_rect()):
                return True
        return False

    def platform_collide(self, player):
        all_collisions = []
        for platform in self.platforms:
            result = platform.collide_with(player.rect)
            if result:
                all_collisions.append((result, platform.rect))
        return all_collisions

    def platform_collide_point(self, left_corner, right_corner):
        for platform in self.platforms:
            if platform.collide_point(left_corner) or platform.collide_point(right_corner):
                return True
        return False
