from config import *


class Npc:
    def __init__(self, world, type, pos):
        self.type = type  # 0 = target, 1 = fellow, 2 = civilian
        self.isActive = True #controls the dead/show
        self.pos = pos
        self.offset = [0, 0]
        self.direction = 0
        if type == 0:
            self.direction = 0
        elif type == 1:
            self.direction = -1

        self.speed = 100
        self.vertical_speed = 0
        self.falling = True
        self.world = world
        self.rect = pygame.Rect(self.pos, PLAYER_SIZE)
        if type == 2:
            self.walk_cycle = Cycle(2, 40)
        else:
            self.walk_cycle = Cycle(5, 4)

        self.cstand = True

    def get_draw_pos(self):
        return pygame.Rect(self.pos[0] - ENV["global_offset"][0] + DEFAULT_CENTER[0],
                           self.pos[1] - ENV["global_offset"][1] + DEFAULT_CENTER[1],
                           PLAYER_SIZE[0], PLAYER_SIZE[1])

    def draw(self):
        pos = (self.get_draw_pos()[0], self.get_draw_pos()[1])
        if self.type == 0:
            # target
            if self.direction > 0:
                draw_at_frame(pos, "target_right", self.walk_cycle.get(),
                              PLAYER_SIZE)
            else:
                draw_at_frame(pos, "target_left", self.walk_cycle.get(),
                              PLAYER_SIZE)
            if self.speed == 0:
                screen.blit(texture_lib["freeze"], (pos[0] + 45, pos[1] - 20))
            elif self.speed < 100:
                screen.blit(texture_lib["sweat"], (pos[0] + 45, pos[1] - 20))
        elif self.type == 1:
            # fellow
            if self.direction > 0:
                draw_at_frame(pos, "fellow_right", self.walk_cycle.get(),
                              PLAYER_SIZE)
            else:
                draw_at_frame(pos, "fellow_left", self.walk_cycle.get(),
                              PLAYER_SIZE)
            if self.speed == 0:
                screen.blit(texture_lib["freeze"], (pos[0] + 45, pos[1] - 20))
            elif self.speed < 100:
                screen.blit(texture_lib["sweat"], (pos[0] + 45, pos[1] - 20))
        elif self.type == 2:
            # civilian
            if self.walk_cycle.get() == 0 and  self.walk_cycle.changed() and random.randint(0, 10) < 3:
                self.cstand = not self.cstand
            if self.cstand:
                draw_at_frame(pos, "civilian_left", 0,
                              PLAYER_SIZE)
            else:
                draw_at_frame(pos, "civilian_right", 0,
                              PLAYER_SIZE)

    def landed(self):
        if self.pos[1] + 1 > GROUND_LEVEL:
            return True
        return self.world.platform_collide_point((self.pos[0], self.pos[1] + PLAYER_SIZE[1] + 1),
                                                  (self.pos[0] + PLAYER_SIZE[0], self.pos[1] + PLAYER_SIZE[1] + 1))

    def AI(self):
        if self.type == 0:
            self.direction = 1
            curr_time = pygame.time.get_ticks()
            if curr_time % 300 == 0:
                ...
                #self.world.npcs.append(Npc(self.world, 1, [self.pos[0], self.pos[1]]))
        elif self.type == 1:
            ...
        elif self.type == 2:
            for aNpc in self.world.npcs:
                if aNpc is not self and aNpc.type == 1:
                    c_f_distance = math.sqrt((aNpc.pos[0]-self.pos[0])**2+(aNpc.pos[1]-self.pos[1])**2)
                    if c_f_distance < PLAYER_SIZE[0]:
                        self.isActive = False
                        ses[3].play_once()
                        self.world.lives -= 1

    def update(self):
        #TODO: update code for fellow IDLE state
        self.AI()
        if self.direction != 0:
            self.pos[0] += ENV["delta_time"] * self.speed * self.direction
            if not self.landed():
                self.falling = True

        if self.falling:
            self.pos[1] += ENV["delta_time"] * self.vertical_speed
            self.vertical_speed += GRAVITY
            if self.pos[1] > GROUND_LEVEL:
                self.falling = False
                self.vertical_speed = 0
                self.pos[1] = GROUND_LEVEL
        collided = self.world.platform_collide(self)
        if collided:
            for collision in collided:
                if collision[0] == "top":
                    self.pos[1] = collision[1].y + collision[1].h + 1
                    self.vertical_speed = 0
                elif collision[0] == "left":
                    if self.type == 1:
                        self.direction *= -1
                    self.pos[0] = collision[1].x + collision[1].w + 1
                elif collision[0] == "right":
                    if self.type == 1:
                        self.direction *= -1
                    self.pos[0] = collision[1].x - PLAYER_SIZE[0] - 1
                elif collision[0] == "bottom":
                    self.pos[1] = collision[1].y - PLAYER_SIZE[1] - 1
                    self.falling = False
                    self.vertical_speed = 0

        ENV["global_offset"] = (self.pos[0], self.pos[1])
        self.rect = pygame.Rect(self.pos, PLAYER_SIZE)







