from config import *
import animation
import math


class Player:
    def __init__(self, world):
        self.pos = [0, 0]
        self.offset = [0, 0]
        self.direction = 0
        self.vertical_speed = 0
        self.falling = True
        self.world = world
        self.rect = self.get_rect()
        self.debuff_ticks_left = 200 # debuff time, if need change, do as well in line 125, 128

        self.walk_cycle = TimedCycle(5, (3, 3, 3, 3, 3), [200,200,200,200,200])

    def get_rect(self):
        return pygame.Rect((self.pos[0] + PLAYER_SIZE[0] / 4, self.pos[1]),
                           (PLAYER_SIZE[0] / 2, PLAYER_SIZE[1]))

    def get_draw_pos(self):
        return SCREEN_WIDTH // 2 - PLAYER_SIZE[0] // 2 + self.offset[0], \
               SCREEN_HEIGHT // 2 - PLAYER_SIZE[1] // 2 + self.offset[1]

    def draw(self):
        pos = self.get_draw_pos()

        if self.direction == -1:
            draw_at_frame(pos, "cop_left", self.walk_cycle.get_frame(), PLAYER_SIZE)
        elif self.direction == 1:
            draw_at_frame(pos, "cop_right", self.walk_cycle.get_frame(), PLAYER_SIZE)
        else:
            draw_at_frame(pos, "cop_right", 0, PLAYER_SIZE)

        mouse_angle = math.atan2(pygame.mouse.get_pos()[1] - pos[1] - PLAYER_SIZE[1] / 2,
                                 pygame.mouse.get_pos()[0] - pos[0] - PLAYER_SIZE[0] / 2)

        pygame.draw.arc(screen, (190, 190, 190), pygame.Rect(
            pos[0] + PLAYER_SIZE[0] / 2 - 250,  # find the center, -250 to center the rectangle
            pos[1] + PLAYER_SIZE[1] / 2 - 250,
            500, 500
        ), -math.pi / 6 - mouse_angle, math.pi / 6 - mouse_angle,
                        width=210)  # start_angle, stop_angle, inward-growing of arc.

        if self.debuff_ticks_left != 200:
            screen.blit(texture_lib["annoyed"], (pos[0] + 45, pos[1] - 20))

    def landed(self):
        if self.pos[1] + 2 > GROUND_LEVEL:
            return True
        return self.world.platform_collide_point((self.pos[0] + PLAYER_SIZE[0] / 4 + 5, self.pos[1] + PLAYER_SIZE[1] + 2),
                                                 (self.pos[0] + PLAYER_SIZE[0] * 3 / 4 - 5,
                                                  self.pos[1] + PLAYER_SIZE[1] + 2))

    def handle_event(self, event):
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_a:
                self.direction -= 1
            elif event.key == pygame.K_d:
                self.direction += 1
            elif event.key == pygame.K_w or event.key == pygame.K_SPACE:
                if not self.falling:
                    self.falling = True
                    self.vertical_speed = -450
                    ses[2].play_once()
        elif event.type == pygame.KEYUP:
            if event.key == pygame.K_a:
                self.direction += 1
            elif event.key == pygame.K_d:
                self.direction -= 1

    def move_camera(self, dx, dy):
        self.offset[0] += dx
        self.offset[1] += dy
        if self.offset[0] != 0:
            self.offset[0] /= 1.1
            if abs(self.offset[0]) < 1:
                self.offset[0] = 0
        if self.offset[1] != 0:
            self.offset[1] /= 1.1
            if abs(self.offset[1]) < 1:
                self.offset[1] = 0

    def move(self, dx, dy):
        self.pos[0] += dx
        self.pos[1] += dy
        if self.world.collide_rect(self):
            if dx:
                dir = dx/abs(dx)
                while self.world.collide_rect(self):
                    self.pos[0] -= dir
            if dy:
                if dy < 0:
                    self.vertical_speed = 0
                dir = dy / abs(dy)
                while self.world.collide_rect(self):
                    self.pos[1] -= dir

    def update(self):
        prev_pos = self.pos[0], self.pos[1]
        if self.direction != 0:
            self.move(ENV["delta_time"] * self.walk_cycle.get_movement() * self.direction, 0)
            self.walk_cycle.tick()
            if not self.falling and self.walk_cycle.changed() and self.walk_cycle.get_frame() == 2:
                self.world.animations.append(animation.WalkDust(self.pos, self.direction != -1))
            if not self.landed():
                self.falling = True

        if self.falling:
            self.move(0, ENV["delta_time"] * self.vertical_speed)
            self.vertical_speed += GRAVITY
            if self.vertical_speed > 500:
                self.vertical_speed = 500
            if self.pos[1] > GROUND_LEVEL:
                self.falling = False
                self.vertical_speed = 0
                self.pos[1] = GROUND_LEVEL
            if self.landed():
                self.falling = False
                self.vertical_speed = 0

        self.move_camera(self.pos[0] - prev_pos[0], self.pos[1] - prev_pos[1])
        ENV["global_offset"] = (self.pos[0] - self.offset[0], self.pos[1] - self.offset[1])
        self.rect = self.get_rect()

        self.move_camera(self.pos[0] - prev_pos[0], self.pos[1] - prev_pos[1])

        pos = self.get_draw_pos()
        mouse_angle = math.atan2(pygame.mouse.get_pos()[1] - pos[1] - PLAYER_SIZE[1] / 2,
                                 pygame.mouse.get_pos()[0] - pos[0] - PLAYER_SIZE[0] / 2)

        if self.debuff_ticks_left != 200:
            if self.debuff_ticks_left == 0:
                self.walk_cycle.movements = [200,200,200,200,200]
                self.debuff_ticks_left = 200
            else:
                self.debuff_ticks_left -= 1

        for aNpc in self.world.npcs:
            if aNpc.isActive:
                if self.rect.colliderect(aNpc.rect):
                    aNpc.isActive = False
                    if aNpc.type == 0:
                        pass
                    elif aNpc.type == 1:
                        if aNpc.speed == 0:
                            ses[4].play_once()
                        else:
                            if self.debuff_ticks_left != 0:
                                self.walk_cycle.movements = [100,100,100,100,100]# debuffed speed
                                self.debuff_ticks_left -= 1
                                ses[0].play_once()

                    elif aNpc.type == 2:
                        aNpc.isActive = True

                curr_dis = math.sqrt((aNpc.pos[0] - self.pos[0]) ** 2 + (aNpc.pos[1] - self.pos[1]) ** 2)
                curr_angle = math.atan2(aNpc.pos[1] - self.pos[1], aNpc.pos[0] - self.pos[0])
                pre_angle_upper = mouse_angle + math.pi / 6  # move clockwise
                pre_angle_lower = mouse_angle - math.pi / 6  # move counter-clockwise

                if pre_angle_upper > math.pi:
                    temp = pre_angle_upper - math.pi
                    post_angle_upper = -math.pi + temp
                    post_angle_lower = pre_angle_lower
                elif pre_angle_lower < -math.pi:
                    temp = pre_angle_lower + math.pi
                    post_angle_lower = math.pi + temp
                    post_angle_upper = pre_angle_upper
                else:
                    post_angle_upper = pre_angle_upper
                    post_angle_lower = pre_angle_lower

                #print(curr_angle, post_angle_upper, post_angle_lower)
                if post_angle_upper < 0 < post_angle_lower:
                    if curr_dis < 250 and (-math.pi <= curr_angle<= post_angle_upper or post_angle_lower <= curr_angle <= math.pi):
                        if aNpc.type == 0 and aNpc.speed != 0:
                            aNpc.speed -= 1  # target speed decrease
                        if aNpc.type == 1 and aNpc.speed != 0:
                            aNpc.speed -= 1  # fellow speed decrease



                elif curr_dis < 250 and post_angle_lower <= curr_angle <= post_angle_upper:
                    if aNpc.type == 0 and aNpc.speed != 0:
                        aNpc.speed -= 1  # target speed decrease
                    if aNpc.type == 1 and aNpc.speed != 0:
                        aNpc.speed -= 1  # fellow speed decrease
